Package: reviser
Type: Package
Title: Analyzing Revisions in Real-Time Time Series Vintages
Version: 0.2.0
Authors@R: c(
    person(given = "Marc", family = "Burri", email = "marc.burri91@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-8974-9090")),
    person(given = "Philipp", family = "Wegmueller",
        email = "philipp.wemueller@seco.admin.ch", role = c("aut", "cph"))
  )
Description: Analyzes revisions in real-time time series vintages. The
    package converts between wide revision triangles and tidy long
    vintages, extracts selected releases, computes revision series,
    visualizes vintage paths, and summarizes revision properties such as
    bias, dispersion, autocorrelation, and news-noise diagnostics. It
    also identifies efficient releases and estimates state-space models
    for revision nowcasting. Methods are based on Howrey (1978)
    <doi:10.2307/1924972>, Jacobs and Van Norden (2011)
    <doi:10.1016/j.jeconom.2010.04.010>, and Kishor and Koenig (2012)
    <doi:10.1198/jbes.2010.08169>.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0)
RoxygenNote: 7.3.3
Imports: dplyr, stats, tidyr, pillar, ggplot2, car, sandwich,
        systemfit, calculus, rlang, scales, tibble, lubridate, KFAS,
        numDeriv, withr
Suggests: tsbox, testthat (>= 3.0.0), knitr, rmarkdown, purrr
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://docs.ropensci.org/reviser/,
        https://github.com/ropensci/reviser
BugReports: https://github.com/ropensci/reviser/issues
NeedsCompilation: no
Packaged: 2026-08-22 17:42:33 UTC; marcburri
Author: Marc Burri [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-8974-9090>),
  Philipp Wegmueller [aut, cph]
Maintainer: Marc Burri <marc.burri91@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-22 18:30:02 UTC
Built: R 4.6.1; ; 2026-08-22 19:00:07 UTC; unix
