# Generated by roxygen2: do not edit by hand

S3method(coef,jvn_model)
S3method(coef,kk_model)
S3method(diagnose,revision_summary)
S3method(fitted,jvn_model)
S3method(fitted,kk_model)
S3method(logLik,jvn_model)
S3method(logLik,kk_model)
S3method(nobs,jvn_model)
S3method(nobs,kk_model)
S3method(plot,jvn_model)
S3method(plot,kk_model)
S3method(plot,tbl_pubdate)
S3method(plot,tbl_release)
S3method(predict,jvn_model)
S3method(predict,kk_model)
S3method(print,jvn_model)
S3method(print,kk_model)
S3method(print,lst_efficient)
S3method(print,revision_summary)
S3method(print,tbl_pubdate)
S3method(print,tbl_release)
S3method(residuals,jvn_model)
S3method(residuals,kk_model)
S3method(states,jvn_model)
S3method(states,kk_model)
S3method(summary,jvn_model)
S3method(summary,kk_model)
S3method(summary,lst_efficient)
S3method(summary,revision_summary)
S3method(summary,tbl_pubdate)
S3method(summary,tbl_release)
S3method(tbl_sum,tbl_pubdate)
S3method(tbl_sum,tbl_release)
S3method(vcov,jvn_model)
S3method(vcov,kk_model)
export(colors_reviser)
export(diagnose)
export(get_days_to_release)
export(get_first_efficient_release)
export(get_first_release)
export(get_fixed_release)
export(get_latest_release)
export(get_nth_release)
export(get_releases_by_date)
export(get_revision_analysis)
export(get_revisions)
export(jvn_nowcast)
export(kk_nowcast)
export(plot_vintages)
export(scale_color_reviser)
export(scale_fill_reviser)
export(states)
export(theme_reviser)
export(validate_vintages)
export(vintages_long)
export(vintages_wide)
importFrom(KFAS,SSMcustom)
importFrom(KFAS,SSModel)
importFrom(calculus,"%diff%")
importFrom(calculus,"%mx%")
importFrom(calculus,"%prod%")
importFrom(calculus,"%sum%")
importFrom(pillar,tbl_sum)
importFrom(stats,coef)
importFrom(stats,fitted)
importFrom(stats,logLik)
importFrom(stats,nobs)
importFrom(stats,predict)
importFrom(stats,residuals)
importFrom(stats,vcov)
